/*
 GBxMAS RW - LED controller
 PCB version: 1.0
 Firmware version: R1
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 15/12/2019
 Last Modified: 5/01/2020
 
 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define LEFT_SEG 0
#define RIGHT_SEG 1

#define LEFT_LED1 PG3
#define LEFT_LED2 PB7
#define LEFT_LED3 PB6
#define LEFT_LED4 PB5
#define LEFT_LED5 PB4
#define LEFT_LED6 PG4
#define LEFT_LED7 PB3
#define LEFT_LED8 PB2
#define LEFT_LED9 PB1
#define LEFT_LED10 PB0
#define LEFT_LED11 PE3
#define LEFT_LED12 PF0
#define LEFT_LED13 PE2
#define LEFT_LED14 PF1

#define RIGHT_LED1 PC3
#define RIGHT_LED2 PC4
#define RIGHT_LED3 PC5
#define RIGHT_LED4 PC7
#define RIGHT_LED5 PC6
#define RIGHT_LED6 PC2
#define RIGHT_LED7 PA6
#define RIGHT_LED8 PA7
#define RIGHT_LED9 PA5
#define RIGHT_LED10 PA4
#define RIGHT_LED11 PA3
#define RIGHT_LED12 PA1
#define RIGHT_LED13 PA2
#define RIGHT_LED14 PA0

#define XMAS_LEDS '#'
#define XMAS_VALUE 0x7690FCD

char receivedBuffer[32];
char receivedChar;
volatile char idleReceivedChar = 0;
volatile uint8_t isIdle = 1;	

// Receive USART data
uint8_t USART_Receive(void) {
	while ( !(UCSRA & (1<<RXC)) && isIdle == 0); // Wait for data to be received
	return UDR; // Get and return received data from buffer
}

// Transmit USART data
void USART_Transmit(unsigned char data) {
	while ( !( UCSRA & (1<<UDRE)) ); // Wait for empty transmit buffer
	UDR = data;
}

// Read 1-256 bytes from the USART 
void usart_read_bytes(int count) {
	for (int x = 0; x < count; x++) {
		receivedBuffer[x] = USART_Receive();
	}
}

// Read the USART until a 0 (string terminator byte) is received
void usart_read_chars(void) {
	int x = 0;
	while (1) {
		receivedBuffer[x] = USART_Receive();
		if (receivedBuffer[x] == 0) {
			break;
		}
		x++;
		if (x >= 10) {
			break;
		}
	}
}

uint8_t next_led_sequence(uint8_t led) {
	if (led <= 14) {
		return led + 15;
	}
	else {
		return led - 14;
	}
}

void setLED(uint8_t number) {
	if (number == 1) {
		PORTG |= (1<<LEFT_LED1);
	}
	else if (number == 2) {
		PORTB |= (1<<LEFT_LED2);
	}
	else if (number == 3) {
		PORTB |= (1<<LEFT_LED3);
	}
	else if (number == 4) {
		PORTB |= (1<<LEFT_LED4);
	}
	else if (number == 5) {
		PORTB |= (1<<LEFT_LED5);
	}
	else if (number == 6) {
		PORTG |= (1<<LEFT_LED6);
	}
	else if (number == 7) {
		PORTB |= (1<<LEFT_LED7);
	}
	else if (number == 8) {
		PORTB |= (1<<LEFT_LED8);
	}
	else if (number == 9) {
		PORTB |= (1<<LEFT_LED9);
	}
	else if (number == 10) {
		PORTB |= (1<<LEFT_LED10);
	}
	else if (number == 11) {
		PORTE |= (1<<LEFT_LED11);
	}
	else if (number == 12) {
		PORTF |= (1<<LEFT_LED12);
	}
	else if (number == 13) {
		PORTE |= (1<<LEFT_LED13);
	}
	else if (number == 14) {
		PORTF |= (1<<LEFT_LED14);
	}
	else if (number == 15) {
		PORTC |= (1<<RIGHT_LED1);
	}
	else if (number == 16) {
		PORTC |= (1<<RIGHT_LED2);
	}
	else if (number == 17) {
		PORTC |= (1<<RIGHT_LED3);
	}
	else if (number == 18) {
		PORTC |= (1<<RIGHT_LED4);
	}
	else if (number == 19) {
		PORTC |= (1<<RIGHT_LED5);
	}
	else if (number == 20) {
		PORTC |= (1<<RIGHT_LED6);
	}
	else if (number == 21) {
		PORTA |= (1<<RIGHT_LED7);
	}
	else if (number == 22) {
		PORTA |= (1<<RIGHT_LED8);
	}
	else if (number == 23) {
		PORTA |= (1<<RIGHT_LED9);
	}
	else if (number == 24) {
		PORTA |= (1<<RIGHT_LED10);
	}
	else if (number == 25) {
		PORTA |= (1<<RIGHT_LED11);
	}
	else if (number == 26) {
		PORTA |= (1<<RIGHT_LED12);
	}
	else if (number == 27) {
		PORTA |= (1<<RIGHT_LED13);
	}
	else if (number == 28) {
		PORTA |= (1<<RIGHT_LED14);
	}
}


void unsetLED(uint8_t number) {
	if (number == 1) {
		PORTG &= ~(1<<LEFT_LED1);
	}
	else if (number == 2) {
		PORTB &= ~(1<<LEFT_LED2);
	}
	else if (number == 3) {
		PORTB &= ~(1<<LEFT_LED3);
	}
	else if (number == 4) {
		PORTB &= ~(1<<LEFT_LED4);
	}
	else if (number == 5) {
		PORTB &= ~(1<<LEFT_LED5);
	}
	else if (number == 6) {
		PORTG &= ~(1<<LEFT_LED6);
	}
	else if (number == 7) {
		PORTB &= ~(1<<LEFT_LED7);
	}
	else if (number == 8) {
		PORTB &= ~(1<<LEFT_LED8);
	}
	else if (number == 9) {
		PORTB &= ~(1<<LEFT_LED9);
	}
	else if (number == 10) {
		PORTB &= ~(1<<LEFT_LED10);
	}
	else if (number == 11) {
		PORTE &= ~(1<<LEFT_LED11);
	}
	else if (number == 12) {
		PORTF &= ~(1<<LEFT_LED12);
	}
	else if (number == 13) {
		PORTE &= ~(1<<LEFT_LED13);
	}
	else if (number == 14) {
		PORTF &= ~(1<<LEFT_LED14);
	}
	else if (number == 15) {
		PORTC &= ~(1<<RIGHT_LED1);
	}
	else if (number == 16) {
		PORTC &= ~(1<<RIGHT_LED2);
	}
	else if (number == 17) {
		PORTC &= ~(1<<RIGHT_LED3);
	}
	else if (number == 18) {
		PORTC &= ~(1<<RIGHT_LED4);
	}
	else if (number == 19) {
		PORTC &= ~(1<<RIGHT_LED5);
	}
	else if (number == 20) {
		PORTC &= ~(1<<RIGHT_LED6);
	}
	else if (number == 21) {
		PORTA &= ~(1<<RIGHT_LED7);
	}
	else if (number == 22) {
		PORTA &= ~(1<<RIGHT_LED8);
	}
	else if (number == 23) {
		PORTA &= ~(1<<RIGHT_LED9);
	}
	else if (number == 24) {
		PORTA &= ~(1<<RIGHT_LED10) ;
	}
	else if (number == 25) {
		PORTA &= ~(1<<RIGHT_LED11);
	}
	else if (number == 26) {
		PORTA &= ~(1<<RIGHT_LED12);
	}
	else if (number == 27) {
		PORTA &= ~(1<<RIGHT_LED13);
	}
	else if (number == 28) {
		PORTA &= ~(1<<RIGHT_LED14);
	}
}

void allLEDOff(void) {
	PORTG &= ~((1<<LEFT_LED1) | (1<<LEFT_LED6));
	PORTF &= ~((1<<LEFT_LED12) | (1<<LEFT_LED14));
	PORTE &= ~((1<<LEFT_LED11) | (1<<LEFT_LED13));
	PORTB &= ~((1<<LEFT_LED2) | (1<<LEFT_LED3) | (1<<LEFT_LED4) | (1<<LEFT_LED5) | (1<<LEFT_LED7) | (1<<LEFT_LED8) | (1<<LEFT_LED9) | (1<<LEFT_LED10));
	PORTC &= ~((1<<RIGHT_LED1) | (1<<RIGHT_LED2) | (1<<RIGHT_LED3) | (1<<RIGHT_LED4) | (1<<RIGHT_LED5) | (1<<RIGHT_LED6));
	PORTA &= ~((1<<RIGHT_LED7) | (1<<RIGHT_LED8) | (1<<RIGHT_LED9) | (1<<RIGHT_LED10) | (1<<RIGHT_LED11) | (1<<RIGHT_LED12) | (1<<RIGHT_LED13) | (1<<RIGHT_LED14));
}

// Setup
void setup(void) {
	if (MCUSR & (1<<WDRF)) { // Check if reset source was the watchdog
		isIdle = 0;
	}
	
	// Turn off watchdog
	MCUSR &= ~(1<<WDRF);
	WDTCR = (1<<WDCE) | (1<<WDE);
	WDTCR = 0;
	
	// Set outputs
	DDRG |= (1<<LEFT_LED1) | (1<<LEFT_LED6);
	DDRF |= (1<<LEFT_LED12) | (1<<LEFT_LED14);
	DDRE |= (1<<LEFT_LED11) | (1<<LEFT_LED13);
	DDRB |= (1<<LEFT_LED2) | (1<<LEFT_LED3) | (1<<LEFT_LED4) | (1<<LEFT_LED5) | (1<<LEFT_LED7) | (1<<LEFT_LED8) | (1<<LEFT_LED9) | (1<<LEFT_LED10);
	DDRC |= (1<<RIGHT_LED1) | (1<<RIGHT_LED2) | (1<<RIGHT_LED3) | (1<<RIGHT_LED4) | (1<<RIGHT_LED5) | (1<<RIGHT_LED6);
	DDRA |= (1<<RIGHT_LED7) | (1<<RIGHT_LED8) | (1<<RIGHT_LED9) | (1<<RIGHT_LED10) | (1<<RIGHT_LED11) | (1<<RIGHT_LED12) | (1<<RIGHT_LED13) | (1<<RIGHT_LED14);
	
	// Setup USART
	UBRRL = 0; // 1Mbps Baud rate
	sbi(UCSRB, RXEN); // Receiver enable
	
	// Turn on interrupts
	sei();
}
